__all__ = ['ClassChecker']

import astroid as _astroid

from .abc_checker import ABChecker as _ABChecker


class ClassChecker(_ABChecker):
    def defines_class(self, class_names=None, require_all=True, components=None, excluding=None):
        definitions = self.class_definitions(True, components, excluding)
        if class_names is None:
            return len(definitions) != 0
        class_names = self._bundle(class_names)
        condition = all if require_all else any
        return condition(self.qual_name(class_name) in definitions for class_name in class_names)

    def initializes_class(self, classes=None, require_all=False, components=None, excluding=None):
        initializations = self.class_initializations(components, excluding)
        if classes is None:
            return len(initializations) != 0
        classes = self._bundle(classes)
        condition = all if require_all else any
        return condition(class_ in initializations for class_ in classes)

    def class_definitions(self, qualified=False, components=None, excluding=None):
        return {class_def.qname() if qualified else class_def.name for class_def in
                self.nodes_of_class(_astroid.ClassDef, components, excluding)}

    def class_initializations(self, components=None, excluding=None):
        calls = set()
        literals = self.nodes_of_class((_astroid.Dict, _astroid.List, _astroid.Set, _astroid.Tuple),
                                       components, excluding)
        calls.update(literal.qname() for literal in literals)
        for call in self.nodes_of_class(_astroid.Call, components, excluding):
            # ToDo - Guard against inference error
            val = next(call.func.infer())
            if isinstance(val, _astroid.ClassDef):
                calls.add(val.qname())
        return calls
