__all__ = ['FunctionChecker']

from collections import defaultdict as _defaultdict

import astroid as _astroid

from src.qchecker.checker.abc_checker import ABChecker as _ABChecker


class FunctionChecker(_ABChecker):
    def calls_function(self, functions=None, require_all=False, components=None, excluding=None):
        calls = self.function_calls(components, excluding)
        if functions is None:
            return len(calls) != 0
        functions = self._bundle(functions)
        condition = all if require_all else any
        return condition(self.qual_name(function) in calls for function in functions)

    def defines_function(self, function_names=None, require_all=True, components=None, excluding=None):
        definitions = self.function_definitions(False, components, excluding) | \
                      self.function_definitions(True, components, excluding)
        if function_names is None:
            return len(definitions) != 0
        function_names = self._bundle(function_names)
        condition = all if require_all else any
        return condition(function_name in definitions for function_name in function_names)

    def function_definitions(self, qualified=False, components=None, excluding=None):
        return {func_def.qname() if qualified else func_def.name for func_def in
                self.nodes_of_class(_astroid.FunctionDef, components, excluding)}

    def function_calls(self, components=None, excluding=None):
        calls = set()
        for call in self.nodes_of_class(_astroid.Call, components, excluding):
            # ToDo - Guard against inference error
            val = next(call.func.infer())
            if isinstance(val, (_astroid.FunctionDef, _astroid.BoundMethod, _astroid.UnboundMethod)):
                calls.add(val.qname())
        return calls

    def count_function_defs(self, components=None, excluding=None):
        return len(self.nodes_of_class(_astroid.FunctionDef, components, excluding))

    def count_function_calls(self, components=None, excluding=None):
        return len(self.nodes_of_class(_astroid.Call, components, excluding))

    def defines_nested_function(self, components=None, excluding=None):
        return self.max_nesting(components, excluding, _astroid.FunctionDef) > 1

    def uses_recursion(self, components=None, excluding=None):
        calls = _defaultdict(set)
        function_defs = self.nodes_of_class(_astroid.FunctionDef, components, excluding)
        for definition in function_defs:
            for call in definition.nodes_of_class(_astroid.Call):
                # ToDo - Guard against inference error
                calls[definition.name].add(next(call.func.infer()).name)

        def _contains_loops(g):
            path = set()

            def visit(vertex):
                path.add(vertex)
                for neighbour in g.get(vertex, ()):
                    if neighbour in path or visit(neighbour):
                        return True
                path.remove(vertex)
                return False

            return any(visit(v) for v in g)

        return _contains_loops(calls)
