from collections import deque

import astroid as _astroid


class NodeVisitor:
    block_types = (_astroid.FunctionDef, _astroid.ClassDef, _astroid.Module)

    def __init__(self, module, in_blocks, excluding):
        self._module = module
        self._block_names = in_blocks
        self._excluding = excluding

    @property
    def blocks(self):
        if self._block_names is None:
            return [self._module]
        nodes = []
        queue = deque((self._module,))
        while len(queue) != 0:
            current = queue.popleft()
            if self._excluding is None or current.qname() not in self._excluding:
                if current.qname() in self._block_names:
                    nodes.append(current)
                else:
                    children = (child for child in current.get_children() if isinstance(child, self.block_types))
                    queue.extend(children)
        return nodes

    def nodes_of_class(self, class_):
        nodes = []
        queue = deque(self.blocks)
        while len(queue) != 0:
            current = queue.popleft()
            if isinstance(current, self.block_types):
                if self._excluding is not None and current.qname() in self._excluding:
                    continue
            if isinstance(current, class_):
                nodes.append(current)
            children = (child for child in current.get_children())
            queue.extend(children)
        return nodes

    def max_nesting(self, node_type):
        def _max_nesting(node, node_type):
            children = []
            for child in node.get_children():
                if not isinstance(child, self.block_types) or \
                        self._excluding is None or \
                        child.qname() not in self._excluding:
                    children.append(child)
            return int(isinstance(node, node_type)) + max((_max_nesting(child, node_type) for child in children),
                                                          default=0)

        return max((_max_nesting(root, node_type) for root in self.blocks), default=0)
