from src.qchecker import FunctionChecker as _FunctionChecker
from src.qchecker.constraints.abc_constraints import ABConstraints as _ABConstraints

__all__ = ['FunctionConstraints']


class FunctionConstraints(_ABConstraints, _FunctionChecker):
    """Defines several methods which check the functions defined and called in the given code"""

    def prevent_function_definition(self, function_names=None, require_all=False, components=None, excluding=None, msg=None,
                                    exit_on_error=False):
        if function_names is None:
            msg = self.std_prevent_msg('any functions', msg, verb='define')
        else:
            msg = self.std_prevent_msg(f'the {function_names} function', msg, verb='define')
        return self.prevent(
            self.defines_function(function_names, require_all, components, excluding),
            msg, exit_on_error)

    def require_function_definition(self, function_names=None, require_all=True, components=None, excluding=None, msg=None,
                                    exit_on_error=False):
        if function_names is None:
            msg = self.std_require_msg('a function', msg, verb='define')
        else:
            msg = self.std_require_msg(f'the {function_names} function', msg, verb='define')
        return self.require(
            self.defines_function(function_names, require_all, components, excluding),
            msg, exit_on_error)

    def prevent_recursion(self, components=None, excluding=None, msg=None, exit_on_error=False):
        msg = self.std_prevent_msg('recursion', msg)
        return self.prevent(self.uses_recursion(components, excluding), msg, exit_on_error)

    def require_recursion(self, components=None, excluding=None, msg=None, exit_on_error=False):
        msg = self.std_require_msg('recursion', msg)
        return self.require(self.uses_recursion(components, excluding), msg, exit_on_error)
